import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

import 'zapt_sdk_flutter_platform_interface.dart';

class MethodChannelZaptSdkFlutter extends ZaptSdkFlutterPlatform {
  @visibleForTesting
  final methodChannel = const MethodChannel('zapt_sdk_flutter');

  @override
  Future<String?> getMapLink(Map<String, dynamic> arguments) async {
    String? mapLink =
        await methodChannel.invokeMethod<String>('getMapLink', arguments);
    if (arguments.isNotEmpty && arguments['options'] != null) {
      Map options = arguments['options'];
      for (MapEntry entry in options.entries) {
        String option = entry.key;
        dynamic value = entry.value;

        if (mapLink != null &&
            !mapLink.contains(option) &&
            option.isNotEmpty &&
            value != null) {
          mapLink += "&$option=${value.toString()}";
        }
      }
    }
    return mapLink;
  }

  @override
  Future<void> requestPermissions() async {
    return await methodChannel.invokeMethod<void>('requestPermissions');
  }

  @override
  Future<void> requestPermissionsBackground(
      Map<String, String> arguments) async {
    return await methodChannel.invokeMethod<void>(
        'requestPermissionsBackground', arguments);
  }

  @override
  Future<void> initialize(String placeId) async {
    return await methodChannel.invokeMethod<void>('initialize', placeId);
  }

  @override
  Future<void> stop() async {
    return await methodChannel.invokeMethod<void>('stop');
  }

  @override
  Future<void> setVisitable(String placeId) async {
    return await methodChannel.invokeMethod('setVisitable', placeId);
  }

  @override
  Future<void> setDisableSyncingForPositioning(bool disable) async {
    return await methodChannel.invokeMethod(
        'setDisableSyncingForPositioning', disable);
  }

  @override
  Future<void> setDisableSyncingForAnalytics(bool disable) async {
    return await methodChannel.invokeMethod(
        'setDisableSyncingForAnalytics', disable);
  }

  @override
  Future<void> addListener(String eventName) async {
    return await methodChannel.invokeMethod('addListener', eventName);
  }

  @override
  Future<void> removeLocationListener() async {
    return await methodChannel.invokeMethod('removeLocationListener');
  }
}
